/*
 *  UltimateIRCd - an Internet Relay Chat Daemon, src/s_help.c
 *
 *  Copyright (C) 2002 by the past and present ircd coders, and others.
 *  Refer to the documentation within doc/authors/ for full credits and copyrights.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 *  $Id: s_err.c,v 1.8 2002/02/08 09:06:39 shadowmaster Exp $
 *
 */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "h.h"
#include "dynconf.h"
#include "numeric.h"

#define HDR(string)	sendto_one(source_p, ":%s 290 %s :%s", me.name, parv[0], string)
#define SND(string)	sendto_one(source_p, ":%s 291 %s :%s", me.name, parv[0], string)
#define FTR(string)	sendto_one(source_p, ":%s 292 %s :%s", me.name, parv[0], string)
#define HLP(string)	sendto_one(source_p, ":%s 293 %s :%s", me.name, parv[0], string)


int
m_help (aClient * client_p, aClient * source_p, int parc, char *parv[])
{
  char *help;

  if (!MyClient (source_p))
    {
      return 0;
    }

  if (parc < 2)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      HLP (" ");
      HLP
	("Please specify a help topic after the /IRCDHELP command from the list below.");
      HLP (" ");
      HLP ("Example: /IRCDHELP CHANMODES");
      HLP (" ");
      HLP ("/IRCDHELP USERCMDS     - A list of User commands");
      HLP ("/IRCDHELP OPERCMDS     - A list of IRC Operator commands");
      HLP
	("/IRCDHELP USERMODES    - A list of UserModes and their functions");
      HLP
	("/IRCDHELP CHANMODES    - A list of ChannelModes and their functions");
      HLP
	("/IRCDHELP OFLAGS       - A list of O:Line flags and their functions");
      HLP
	("/IRCDHELP ABOUT        - Information about the Ultimate Help System");
      HLP (" ");
      sendto_one (source_p,
		  ":%s 292 %s :If you require any further assistance, please join %s.",
		  me.name, parv[0], HELPCHAN);
      FTR
	("\2~~~~~~~~~~~~~~~~~~~~~~~~~~\2");

      return 0;
    }

  help = parv[1];

  if (ircncmp (help, "USERCMDS", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Below is a list of User Commands currently available. Type \'/IRCDHELP <command>\'");
      SND ("for more information on the command.");
      SND (" ");
      SND ("ADMIN    AWAY     CHANNEL  CREDITS  COPYRIGHT");
      SND ("DCCALLOW HELP     INFO     INVITE   IRCDHELP");
      SND ("IRCOPS   ISON     JOIN     KICK     KNOCK");
      SND ("LINKS    LIST     LUSERS   MAKEPASS MAP");
      SND ("MODE     MOTD     NICK     NOTICE   PART");
      SND ("PING     PONG     PRIVATE  RULES    SETTINGS");
      SND ("SILENCE  STATS    TIME     TRACE    TOPIC");
      SND ("USERHOST USERS    VERSION  WATCH    WHO");
      SND ("WHOIS    QUIT");
    }
  else if (ircncmp (help, "ADMIN", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Displays the given servers administrative information as set by its A line..");
      SND (" ");
      SND ("Example: /ADMIN [server]");
      SND ("         /ADMIN whiterose.shadow-realm.org");
      SND ("         /ADMIN whiterose.*");
    }
  else if (ircncmp (help, "AWAY", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND ("Marks you as away on the server.");
      SND ("A line in your whois shows why you are away.");
      SND (" ");
      SND ("Example: /AWAY Currently writeing the help");
    }
  else if (ircncmp (help, "NICK", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Changes your \"online identify\" (nickname) on the server. All channels that you are");
      SND ("a member of will be alerted of your nickname change.");
      SND (" ");
      SND ("Example: /NICK <newnickname>");
      SND ("         /NICK Acidic32");
    }
  else if (ircncmp (help, "IRCOPS", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Shows you all connected IRCops on the network that are availible for help");
      SND ("and their operator level.");
      SND (" ");
      SND ("Example: /IRCOPS");
    }
  else if (ircncmp (help, "WHOIS", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Shows you information about the user you specify. The information includes nickname,");
      SND ("username, hostname, real name, etc.");
      SND (" ");
      SND ("Example: /WHOIS Acidic32");
      SND ("         /WHOIS ShadowMaster");

    }
  else if (ircncmp (help, "WHO", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Searches visible (-i) user information for the information provided.");
      SND
	("When used on a channel, it will give list of all the visible users on");
      SND ("the specified channel.");
      SND ("To see a list of all supported /WHO parameters see /WHO ?");
      SND (" ");
      SND (" Example: /WHO *.shadow-realm.org");
      SND ("          /WHO Acidic32");
      SND ("          /WHO #ShadowRealm");

    }
  else if (ircncmp (help, "RULES", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Reads the Server Rules file (ircd.rules) and displays them to the user.");
      SND (" ");
      SND ("Example: /RULES");
      SND ("         /RULES ShadowRealm.*");

    }
  else if (ircncmp (help, "IDENTIFY", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("A services alias which allows you to identify your nickname with NickServ or channel");
      SND ("with ChanServ without anyone being able to \'snoop\'.");
      SND (" ");
      SND ("Example: /IDENTIFY <password> - For identifying with NickServ");
      SND
	("         /IDENTIFY <channel> <password> - For identifying with ChanServ");

    }
  else if (ircncmp (help, "ISON", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Used to determine whether the specified user is currently connected to IRC.");
      SND (" ");
      SND ("Example: /ISON Acidic32");

    }
  else if (ircncmp (help, "JOIN", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Used to enter the specified channel(s). Occupants of the specified channel(s) will be");
      SND ("notified of your arrival.");
      SND (" ");
      SND ("Example: /JOIN #ShadowRealm");
      SND ("         /JOIN #ShadowRealm,#WorldVillage,#IRCd");

    }
  else if (ircncmp (help, "PART", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Used to part the specified channel(s). Occupants of the specified channel(s) will be");
      SND ("notified or your departure.");
      SND (" ");
      SND ("Example: /PART #ShadowRealm");
      SND ("         /PART #ShadowRealm,#WorldVillage,#IRCd");

    }
  else if (ircncmp (help, "NAMES", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Provides you with a list of all clients which currently occupy the specified channel.");
      SND (" ");
      SND ("Example: /NAMES #ShadowRealm");

    }
  else if (ircncmp (help, "USERHOST", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND ("Returns the hostname of the speicifed user.");
      SND (" ");
      SND ("Example: /USERHOST Acidic32]");

    }
  else if (ircncmp (help, "LUSERS", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Displays the local and global user statistics. The information includes maximum local");
      SND ("user count, maximum global user count, etc.");
      SND (" ");
      SND ("Example: /LUSERS");
      SND ("         /LUSERS LadyHawke.*");

    }
  else if (ircncmp (help, "MAP", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Displays a graphical map of the IRC Network, and the amount of users on each server.");
      SND (" ");
      SND ("Example: /MAP");

    }
  else if (ircncmp (help, "QUIT", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Disconnects you from the IRC Network. Clients occupying the channels you are in will be");
      SND
	("notified of your departure, aswell as IRC Operators on your local server.");
      SND (" ");
      SND ("Example: /QUIT");
      SND ("         /QUIT Goodbye everyone!");

    }
  else if (ircncmp (help, "STATS", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Reports statistical information about the specified subject. Some characters are limited");
      SND ("to IRC Operators.");
      SND (" ");
      SND ("Example: /STATS ?");
      SND ("         /STATS w");
      SND ("         /STATS k");

    }
  else if (ircncmp (help, "LINKS", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND ("Displays a list of IRC Servers linked to the network.");
      SND (" ");
      SND ("Example: /LINKS");

    }
  else if (ircncmp (help, "ADMIN", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Displays information about the Server Administrator. The information includes the");
      SND ("administrators nickname, real name and e-mail address.");
      SND (" ");
      SND ("Example: /ADMIN");

    }
  else if (ircncmp (help, "TIME", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND ("Displays the current Time and Date of the IRC Server.");
      SND (" ");
      SND ("Example: /TIME");

    }
  else if (ircncmp (help, "INVITE", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Sends the specified user an invitation to join the channel. You must be a channel");
      SND ("operator or half-operator to be able to use this function.");
      SND (" ");
      SND ("Example: /INVITE Acidic32 #ShadowRealm");

    }
  else if (ircncmp (help, "KICK", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Kicks (removes) the specified user from the channel. You must be a channel operator or");
      SND
	("half-operator to use this function. If no reason is specified, reason becomes \'no reason");
      SND ("specified\'.");
      SND (" ");
      SND ("Example: /KICK #ShadowRealm ShadowMaster");
      SND ("         /KICK #ShadowRealm Acidic32 bad language");

    }
  else if (ircncmp (help, "AWAY", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Sets your online status to \'away\' because of the specified reason.");
      SND (" ");
      SND ("Example: /AWAY I am currently away right now!");

    }
  else if (ircncmp (help, "WATCH", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("WATCH is a new type of \'notify\' system. The server will notify you whenever a client");
      SND
	("in your WATCH list connects or disconnects from IRC. Please note, your WATCH list _DOES_NOT_");
      SND
	("remain between IRC sessions. You will need to re-build it whenever you reconnect to IRC.");
      SND (" ");
      SND ("Example: /WATCH +nick1 +nick2 +nick3");
      SND ("         /WATCH -nick1 -nick3");

    }
  else if (ircncmp (help, "LIST", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Displays a list of channels currently open. The information includes the current number of");
      SND ("channel users, and the channel topic.");
      SND (" ");
      SND
	("UltimateIRCd also supports \'search options\'. To use a search option, you must prefix the LIST");
      SND
	("command with \'/QUOTE\' to avoid your IRC Client interpreting the command. If you do not specify");
      SND
	("a search option, the server will display the full, unfiltered channel list.");
      SND (" ");
      SND ("Search Options:");
      SND (" ");
      SND (" > number      - List channels with more than <number> of users");
      SND (" < number      - List channels with less than <nubmer> of users");
      SND
	(" C > number    - List channels created between now and <number> minutes ago");
      SND
	(" C < number    - List channels created earlier than <number> minutes ago");
      SND
	(" T > number    - List channels whose topics are older than <number> minutes");
      SND
	(" T < number    - List channels whose topics are newer than <number> minutes");
      SND (" *mask*        - List all channels that match *mask*");
      SND (" !*mask*       - List all channels that do not match *mask*");
      SND (" ");
      SND ("Example: /LIST");
      SND ("         /QUOTE LIST *IRCd*");
      SND ("         /QUOTE LIST < 30");

    }
  else if ((ircncmp (help, "PRIVMSG", 9) == 0)
	   || (ircncmp (help, "NOTICE", 9) == 0))
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Used to sent text to clients and channels. PRIVMSG is used internally by IRC Clients");
      SND ("for /MSG. UltimateIRCd also supports three additional formats.");
      SND (" ");
      SND ("Example: /MSG Acidic32 Hello. Im bored");
      SND ("         /NOTICE Acidic32 hey there! what are you doing?");
      SND
	("         /NOTICE @#channel :text    - Sends to all channel operators in #channel");
      SND
	("         /NOTICE %#channel :text    - Sends to all half operators in #channel");
      SND
	("         /NOTICE +#channel :text    - Sneds to all voiced users in #channel");
      SND (" ");
      SND
	("Note: The PRIVMSG syntax is the same as the NOTICE/MSG syntax above.");

    }
  else if (ircncmp (help, "VHOST", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Provided you log in using the correct username and password, your hostname");
      SND
	("will be masked using the virtual host (vhost) provided by the IRC Server.");
      SND (" ");
      SND ("Example: /VHOST <login> <password>");
      SND ("         /VHOST billgates ilovelinux");

    }
  else if (ircncmp (help, "TOPIC", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Used to either display the current topic of the specified channel, or to change the");
      SND ("of the specified channel to the specified topic.");
      SND (" ");
      SND ("Example: /TOPIC #ShadowRealm");
      SND
	("         /TOPIC #ShadowRealm UltimateIRCd 3.0.0 has been released!");

    }
  else if (ircncmp (help, "MODE", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Sets a mode upon a channel or user. For a full list of UserModes and their function,");
      SND
	("type \'/IRCDHELP ?USERMODES\'. For a full list of ChannelModes and their function, type");
      SND ("\'/IRCDHELP ?CHANMODES\'.");
      SND (" ");
      SND ("Example: /MODE acidic32 +x-i");
      SND ("         /MODE #ShadowRealm +h-o Acidic32 Acidic32");

    }
  else if (ircncmp (help, "INFO", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND ("Displays information about the IRC Server.");
      SND (" ");
      SND ("Example: /INFO");
      SND ("         /INFO LadyHawke.*");

    }
  else if (ircncmp (help, "VERSION", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Displays the version information about the current IRC Server software being used.");
      SND (" ");
      SND ("Example: /VERSION");
      SND ("         /VERSION LadyHawke.*");

    }
  else if (ircncmp (help, "MOTD", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Displays the contents of the servers Message of the Day file (ircd.motd)");
      SND (" ");
      SND ("Example: /MOTD");
      SND ("         /MOTD LadyHawke.*");

    }
  else if (ircncmp (help, "WHOWAS", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Displays previous WHOIS information for specified users who are no longer connected");
      SND ("to IRC.");
      SND (" ");
      SND ("Example: /WHOWAS Acidic32");
      SND ("         /WHOWAS ShadowMaster");

    }
  else if (ircncmp (help, "MKPASSWD", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Encrypts the specified text so that it can be added directly to the ircd.conf file");
      SND ("where crypt is enabled.");
      SND (" ");
      SND ("Example: /MAKEPASS rtfm");
      SND ("         /MAKEPASS myoperpassislreallylame");

    }
  else if (ircncmp (help, "PING", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("The PING command is used to test the presence of an active client or server at");
      SND
	("the other end of the connection. Servers send a PING message at regular intervals");
      SND
	("if no other activity is detected from the connection. If the connection fails to");
      SND
	("reply to the PING message within a sertain amount of time, the connection is");
      SND
	("closed. A PING message may be sent even if the connection is active.");
      SND (" ");
      SND ("Note: This function is not the same as the CTCP PING command.");
      SND (" ");
      SND ("Example: PING <server1> <server2>");
      SND ("         PING LadyHawke.*");
      SND ("         PING Acidic32");
      SND ("         PING Acidic32 LadyHawke.*");

    }
  else if (ircncmp (help, "PONG", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("The PONG command is used to reply to a PING message. If the <server2> parameter");
      SND
	("is given, this message will be forwarded to the given target. The <server1>");
      SND
	("parameter is the name of the entity who has responded to the PING message using");
      SND ("the PONG message.");
      SND (" ");
      SND ("Example: PONG <server1> <server2>");
      SND
	("         PONG DarkFyre.* Hub.*     - PONG message from DarkFyre.* to Hub.*");

    }
  else if (ircncmp (help, "OPERCMDS", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Below is a list of IRC Operator Commands currently available. Type \'/IRCDHELP ?<command>\'");
      SND ("for more information on the command.");
      SND (" ");
      SND ("OPER      REHASH   AKILL    WALLOPS");
      SND ("RESTART   RAKILL   TRACE");
      SND ("ADCHAT    DIE      KILL     ZLINE");
      SND ("SQUIT     CONNECT  GLOBOPS  FMODE");
      SND ("KLINE     LOCOPS   UNKLINE  NETGLOBAL");
      SND ("SETHOST   FJOIN");

    }
  else if (ircncmp (help, "SETHOST", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND ("Allows you to change your host or another users host");
      SND (" ");
      SND (" Example: /SETHOST Acidic32 i.code.for.shadow-realm.org");

    }
  else if (ircncmp (help, "FJOIN", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND ("Allows an Operator to force entry into the specified channel");
      SND
	("This override any bans and other limits by modes in place on the channel.");
      SND (" ");
      SND (" Example: /FJOIN #ShadowRealm");

    }

  else if (ircncmp (help, "NETGLOBAL", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND ("This allows you to send Global IRCop messages.");
      SND ("Useful for network infomation broadcasting.");
      SND (" ");
      SND (" Example: /NETGLOBAL :Hi ALl Network Opers!!");

    }

  else if (ircncmp (help, "FMODE", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("This allows IRCops with the correct permisions to Force Channel Modes.");
      SND ("Useful for setting Channel Modes incase of floods/takeovers.");
      SND (" ");
      SND (" Example: /FMODE #ShadowRealm +R");
      SND ("          /FMODE #ShadowRealm +o Acidic32");
      SND ("          /FMODE #ShadowRealm +Ro Acidic32");

    }
  else if (ircncmp (help, "OPER", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Sets IRC Operator privileges upon the user provided they \'OperUp\' correctly.");
      SND (" ");
      SND ("Example: /OPER <name> <password>");
      SND ("         /OPER acidic32 rtfm");

    }
  else if (ircncmp (help, "REHASH", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("This command forces the IRC Server to re-load *ALL* the configuration files. If");
      SND
	("a server is specified, the specified server will re-load its configuration files.");
      SND (" ");
      SND ("Example: /REHASH");
      SND ("         /REHASH Ladyhawke.*");

    }
  else if (ircncmp (help, "WALLOPS", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Sends a message to all users who have usermode +w. Only IRC Operators may send WALLOPS,");
      SND
	("however any user can set themselves +w and view WALLOP messages.");
      SND (" ");
      SND ("Example: /WALLOPS Hello everyone who has mode +w set!");

    }
  else if (ircncmp (help, "RESTART", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Forces the IRC Server to flush *ALL* of its connections and hash tables.");
      SND ("Requires password if X:Line is present.");
      SND (" ");
      SND ("Example: /RESTART");

    }
  else if (ircncmp (help, "ADCHAT", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Used to send a message to all Server Administrators and Co-Server");
      SND
	("Administrators connected to IRC. Co-Server Administrators and up only.");
      SND (" ");
      SND ("Example: /ADCHAT :Hello all the Admins!");

    }
  else if (ircncmp (help, "DIE", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Used to terminate (shut down) the IRC Server. Requires password if X:Line");
      SND ("is present.");
      SND (" ");
      SND ("Example: /DIE");

    }
  else if (ircncmp (help, "KILL", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND ("Removes the specified user(s) from the server.");
      SND (" ");
      SND ("Example: /KILL <user1>,<user2>,<user3> <reason>");
      SND ("         /KILL Acidic32,ShadowMaster,B3N goodbye lamers!");

    }
  else if (ircncmp (help, "ZLINE", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Bans all users matching the specified IP address. Note: The ban is *NOT* global.");
      SND (" ");
      SND
	("Example: /ZLINE 127.0.0.1 :We do not allow connections from localhost");

    }
  else if (ircncmp (help, "SQUIT", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND ("Terminates the connection between an IRC Server on a network.");
      SND (" ");
      SND ("Example: /SQUIT HiTech.*");

    }
  else if (ircncmp (help, "CONNECT", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Links an IRC Server to the server you are currently on. UltimateIRCd also allows");
      SND ("\'remote\' connections.");
      SND (" ");
      SND ("Example: /CONNECT <server>");
      SND ("         /CONNECT <server1> <port> <server2>");
      SND ("         /CONNECT Hub.*");
      SND ("         /CONNECT MadChat.* 7000 Hub.*");

    }
  else if (ircncmp (help, "GLOBOPS", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Sends a message to all IRC Operators connected to the network. Unlike WALLOPS,");
      SND
	("the messages are only viewable by IRC Operators. Normally used for important");
      SND ("network information.");
      SND (" ");
      SND ("Example: /GLOBOPS :Them damn flooders are back!");

    }
  else if (ircncmp (help, "KLINE", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Bans the specified user@host from the IRC Server. Note: The ban is *NOT* global.");
      SND (" ");
      SND ("Example: /KLINE *@*.btinternet.com Abuse");

    }
  else if (ircncmp (help, "LOCOPS", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Similar to GLOBOPS, accept only IRC Operators local to your server receive");
      SND ("the message.");
      SND (" ");
      SND ("Example: /LOCOPS :Hello to all the IRC Operators on my server!");

    }
  else if (ircncmp (help, "UNKLINE", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND ("Used to remove a K:Line from the server.");
      SND (" ");
      SND ("Example: /UNKLINE *@*.shadow-realm.org");

    }
  else if (ircncmp (help, "TRACE", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND ("Used to find out what servers are connected to what.");
      SND (" ");
      SND ("Example: /TRACE Shadow.*");

    }
  else if (ircncmp (help, "USERMODES", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Below is a list of the User Modes currently available, along with their function.");
      SND (" ");
      SND ("Modes availible to all users:");
      SND (" ");
      SND ("D - Has seen /DCCALLOW warning message");
      SND ("i - Invisible (Hidden in /WHO)");
      SND ("k - Can see server originating kill notices");
      SND ("s - Can see server notices");
      SND ("w - Can see /WALLOPS notices");
      SND (" ");
      SND ("Modes which are server controlled:");
      SND (" ");
      SND ("r - Registered nickname");
      SND ("S - Services client");
      SND ("x - Real host is hidden");
      SND (" ");
      SND
	("Modes restricted to operators, most additionally restricted by OLine flags:");
      SND (" ");
      SND ("c - Can see connect notices");
      SND ("C - Can see /CHATOPS notices");
      SND ("d - Can see debug notices");
      SND ("e - Can see blocked DCC notices");
      SND ("f - Can see exess flood notices");
      SND ("F - Can see global connect notices");
      SND ("g - Can see /GLOBOPS notices");
      SND ("m - Can see spam warning notices");
      SND ("n - Can see /NETINFO and routing notices");
      SND ("p - Protected irc operator");
      SND ("W - Can see /WHOIS notices");
      SND ("y - Can see \"SPY\" notices");
      SND ("h - Help operator");
      SND (" ");
      SND ("Modes designating IRC operator levels:");
      SND (" ");
      SND ("O - Local irc operator");
      SND ("o - Global irc operator");
      SND (" ");
      SND ("Modes designating services access levels:");
      SND (" ");
      SND ("a - Services operator");
      SND ("P - Services administrator");
      SND ("Z - Services root administrator");
      SND (" ");
      SND
	("To set a usermode on yourself, type: \'/mode <yournick> <modes>\'");
      SND ("Example: /mode Acidic32 +s");
      SND ("         /mode Acidic32 -s");

    }
  else if (ircncmp (help, "CHANMODES", 9) == 0)
    {
      HDR ("~~~~~~~~ UltimateIRCd Help ~~~~~~~~");
      SND (" ");
      SND
	("Below is a list of the Channel Modes currently available, along with their function.");
      SND (" ");
      SND
	("p - Private (will not show on /LIST; cannot join unless invited)");
      SND ("s - Secret (will not show on /LIST");
      SND ("i - Invite Only (cannot join unless /INVITE'd)");
      SND ("K - No Knocks (disables /KNOCK for specific channel)");
      SND ("m - Moderated (only op/halfop/voice users may speak)");
      SND ("n - No External Messages");
      SND ("t - Topic Lock (only op/halfop users may change topic)");
      SND ("r - Registered Channel");
      SND ("R - Registered Nicks Only (only registered nicknames may join)");
      SND ("O - IRC Operators Only");
      SND
	("a <nickname> - Channel Admin (cannot be deopped/halfopped/voiced/kicked");
      SND ("               from channel accept by services)");
      SND ("o <nickname> - Channel Operator (can toggle channel settings)");
      SND
	("h <nickname> - Half Operator (can toggle various channel settings)");
      SND
	("v <nickname> - Voiced User (can speak even when banned or channel is +m)");
      SND
	("l <number> - User Limit (only specified amount of users may join channel)");
      SND
	("k <key> - 'Lock' Channel (must specify correct key to join channel)");
      SND ("N - Stops /INVITE's to your channel");
      SND
	("b <nick!user@host> - Ban (no users matching nick!user@host may join channel)");
      SND
	("e <nick!user@host> - Exception Ban (users matching nick!user@host may join channel");
      SND ("                     even when banned)");
      SND (" ");
      SND
	("To set a channel mode, type: \'/mode #channel <mode> [parameters]\'");
      SND ("Example: /mode #shadowrealm +nto Acidic32");
      SND ("         /mode #acidroom +b *shadow*!*@*.shadow-realm.org");

    }
  else if (ircncmp (help, "OFLAGS", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      SND (" ");
      SND
	("Below is a list of the O:Line Flags currently available, along with their function.");
      SND (" ");
      SND (" o = Local oper");
      SND (" O = Global operator");
      SND (" ");
      SND (" a = Services operator");
      SND (" P = Services administrator");
      SND (" Z = Services root administrator");
      SND (" ");
      SND (" j = Guest administrator");
      SND (" J = Server co administrator");
      SND (" A = Server administrator");
      SND (" t = Net co administrator");
      SND (" T = Network administrator");
      SND (" ");
      SND (" b = Access to do /kline");
      SND (" B = Access to do /unkline");
      SND (" z = Access to do /zline");
      SND (" c = Access to do local /connects and /squits");
      SND (" C = Access to do remote /connects and /squits");
      SND (" k = Access to do local /kills");
      SND (" K = Access to do global /kills");
      SND (" r = Access to do /rehash server");
      SND (" R = Access to do /restart");
      SND (" D = Access to do /die");
      SND (" ");
      SND (" p = Oper can set +p");
      SND (" h = Oper can set +h");
      SND (" u = oper can set +c");
      SND (" d = Oper can set +d");
      SND (" f = Oper can set +f");
      SND (" y = Oper can set +y");
      SND (" ");
      SND (" l = Oper can send /locops");
      SND (" g = Oper can send /chatops");
      SND (" w = oper can send /wallops");
      SND (" G = Oper can send /globops");
      SND
	(" n = Oper can send local server notices (/notice $servername message)");
      SND
	(" N = Oper can send global notices (/notice $*.network.net message)");
      SND (" ");
      SND (" E = Oper can edit the config");
    }
  else if (ircncmp (help, "ABOUT", 9) == 0)
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~");
      SND (" ");
      SND
	("The Ultimate Help System was coded by Acidic32 (aka Daniel Moss)");
      SND
	("as a replacement for documentation files. For more information on the IRCd");
      SND ("itself, type \'/INFO\'");
      SND (" ");
      SND
	("The UltimateIRCd Help System - Copyright (C) Daniel Moss, UltimateIRCd, Inc.");
    }
  else
    {
      HDR
	("\2~~~~~~~~ UltimateIRCd Help ~~~~~~~~\2");
      HLP (" ");
      HLP
	("Please specify a help topic after the /IRCDHELP command from the list below.");
      HLP (" ");
      HLP ("Example: /IRCDHELP CHANMODES");
      HLP (" ");
      HLP ("/IRCDHELP USERCMDS     - A list of User commands");
      HLP ("/IRCDHELP OPERCMDS     - A list of IRC Operator commands");
      HLP
	("/IRCDHELP USERMODES    - A list of UserModes and their functions");
      HLP
	("/IRCDHELP CHANMODES    - A list of ChannelModes and their functions");
      HLP
	("/IRCDHELP OFLAGS       - A list of O:Line flags and their functions");
      HLP
	("/IRCDHELP ABOUT        - Information about the Ultimate Help System");
    }

  HLP (" ");
  sendto_one (source_p,
	      ":%s 292 %s :If you require any further assistance, please join %s.",
	      me.name, parv[0], HELPCHAN);
  sendto_one (source_p,
	      ":%s 292 %s :Please note that /IRCDHELP is work in progress and can be inaccurate or blatantly wrong until final 3.0.0 release.",
	      me.name, parv[0]);
  FTR
    ("\2~~~~~~~~~~~~~~~~~~~~~~~~~~\2");
  return 1;
}
